<?php
/**
 * Plugin Qcm (addon for plugin Adm_redacteurs)
 * Version  : 1.5 (2021/01/03)
 * Compatibility : Guppy v5.x et v6.00.00 et Adm_redacteurs v6.14+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

require_once ADMR_HELPERS_PATH.'/add_record_helper.php';

class Add_qcm_questions_helper extends Add_record_helper
{
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    protected $CI;                       // object global
    protected $_lang = array();          // Données de langue
    protected $_config = array();        // Données de configuration  
    protected $_record_id = 0;           // Identifiant de l'enregistrement
    protected $_record_data = array();   // Tableau contenant les données de l'enregistrement
    protected $_form_params = array();   // Tableau contenant les données de l'enregistrement postées dans le formulaire
 	protected $_form_errors = array();	 // Tableau contenant les messages d'erreur dans le formulaire
    protected $_form_validation = FALSE; // Validation du formulaire (TRUE ou FALSE)

    protected $_doc_infos = array('name' => 'qcm_questions', 'type' => 'qcm'); // Tableau contenant les informations du document (nom, type)
    protected $_writer_rights = array(); // Droits du rédacteur

//    protected $_qcm_id = 0; // Identifiant du qcm
    
    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */    
    public function __construct($writer_rights)
    {
        $this -> CI =& Controller :: get_instance();

        $this -> _writer_rights = $this -> _set_writer_rights($writer_rights);      

        $this -> CI -> load_addon_helper('qcm', 'plugin_menu_for_qcm_helper');
        $this -> CI -> load_addon_helper('qcm', 'qcm_infos_helper');
       
        $this -> CI -> load_addon_model('qcm', 'qcm_ids_model');
        $this -> CI -> load_addon_model('qcm', 'qcm_questions_model');
        $this -> CI -> load_addon_model('qcm', 'writer_qcm_model', array('id' => $this -> _get_writer_id()));
        
        $this -> CI -> load_addon_lang('qcm');
        $this -> CI -> load_addon_config('qcm');
        
        $this -> _lang = $this -> CI -> get_lang();
        $this -> _config = $this -> CI -> get_config();        

        $this -> _qcm_id = Var_helper :: import('qcm_id', '', 0, array('trim'), array(array('Var_helper', 'is_digit')));
        
        parent :: __construct();

        /**
         * Controle de sécurité nécessaire si ce n'est pas l'admin qui est aux commandes
         */
        if (! ADMR_IS_ADMIN && (! $this -> _is_doc_allowed($this -> _record_id) || ! $this -> _is_qcm_allowed($this -> _form_params['qcm_id']))) {
            Url_helper :: redirect('display_error&type=disallowed');
        }
        //       
    }

    /*
    |--------------------------------------------------------------------------
    | Methods
    |--------------------------------------------------------------------------
    */    

    /**
	 * Index
     * 
	 * @access public
     * @return void
	 */	
    public function index()
    {
        parent :: index();
    }

	// --------------------------------------------------------------------

    /*
     * Définit et renvoie le titre de la page 
     *
     * @param array Données à faire passer à la vue
     * @return void
     */        
    public function set_title_page()
    {
        return ucfirst($this -> _lang['add_qcm_question']);
    }

	// --------------------------------------------------------------------

	/**
	 * définit et renvoie les ajouts (js et css) dans l'entête de la page
     * 
     * @access public
	 * @return string Chaine html contenant les ajouts js et css
	 */	
    public function set_header_addons()
    {
        $js = '';
        $js .= '<script type="text/javascript" src="'.ADMR_JS_PATH.'/placeholder/jquery.placeholder.min.js"></script>'."\n";
        $js .= '<script type="text/javascript" src="'.ADMR_JS_PATH.'/datetimepicker/jquery.datetimepicker.min.js"></script>'."\n";
        $js .= '<script type="text/javascript" src="'.ADMR_JS_PATH.'/form-validator/jquery.form-validator.min.js"></script>'."\n";
        $js .= '<script type="text/javascript">$.formUtils.loadModules("module_add_doc", "'.ADMR_JS_PATH.'/form-validator/my_modules/");</script>'."\n";
        $js .= '<script type="text/javascript" src="'.ADMR_JS_PATH.'/add_doc.js"></script>'."\n";
       
        $css = '';
        $css .= '<link rel="stylesheet" type="text/css" href="'.ADMR_CSS_PATH.'/datetimepicker/jquery.datetimepicker.css" />'."\n";
        $css .= '<link rel="stylesheet" type="text/css" href="'.ADMR_CSS_PATH.'/form_validator.css" />'."\n";
        $css .= '<link rel="stylesheet" type="text/css" href="'.ADMR_CSS_PATH.'/add_doc.css" />'."\n";
        $css .= '<link rel="stylesheet" type="text/css" href="'.QCM_CSS_PATH.'/qcm.css" />'."\n";

        return $js.$css;
    }
    
	// --------------------------------------------------------------------

	/**
	 * définit et renvoie la description de la page ainsi que les recommendations
     * 
	 * @return string infos de la page
	 */	
    public function set_infos_page_content()
    {		
        return '';
    }

	// --------------------------------------------------------------------

    /*
     * Définit le contenu du menu qui est affiché dans le bandeau en haut de la page
     *
     * @return array Tableau contenant les items du menu (vide par défaut)
     */
    public function set_menu_plugin_items()
    {        
        if (ADMR_IS_ADMIN) {
            return Plugin_menu_for_qcm_helper :: set_admin_items('qcm', $this -> _lang, $this -> _config, $this -> _writer_rights, $options = array('qcm_id' => $this -> _qcm_id));
        } else {
            return Plugin_menu_for_qcm_helper :: set_public_items('qcm', $this -> _lang, $this -> _config, $this -> _writer_rights, $options = array('qcm_id' => $this -> _qcm_id));
        }
    }

	// --------------------------------------------------------------------

    /**
     * Définit le contenu des informations du plugin qui sont affichées en bas de la page
     *
     * @return array Tableau contenant les informations du plugin sous la forme array('version' => , 'licence' => )
     */
    public function set_infos_plugin()
    {
        return Qcm_infos_helper :: set_plugin_infos($this -> _lang);
    }

	// --------------------------------------------------------------------

	/**
	 * définit les droits du rédacteur
     * 
     * @access protected
     * @param array Tabelau définissant les droits du rédacteur (vide par défaut) 
	 * @return array Tableau contenant les droits du rédacteur
	 */	
    protected function _set_writer_rights($writer_rights = array())
    {
        settype($writer_rights, 'array');
        
        return $this -> _writer_rights = $writer_rights;        
    }

	// --------------------------------------------------------------------

    /**
	 * renvoie l'identifiant du rédacteur concerné
     * 
	 * @access public
     * @return void
	 */	
    protected function _get_writer_id()
    {
        return $this -> _writer_rights['id'];    
    }

	// --------------------------------------------------------------------

	/**
	 * construit et retourne le contenu de l'attribut action du formulaire
     * 
     * @access protected
     * @param mixed Options du menu select
	 * @return void
	 */	
    protected function _build_form_action_content()
    {
        return Url_helper :: build_uri(array(
            'id'      => $this -> _record_id,
//            'qcm_id' => $this -> _qcm_id,
            'idpg'    => Var_helper :: import('idpg', '', 1, array('trim'), array(array('Var_helper', 'is_digit'))),
            'doctype' => trim(Var_helper :: import('doctype', '', NULL, array('trim'), array(array('Var_helper', 'is_alpha_num_dash'))), '-'),
            'sort'    => Var_helper :: import('sort', '', $this -> _config['sort_option_by_default'], array('trim'), array(array('Var_helper', 'is_alpha_num_dash'))),
        ));
    }

	// --------------------------------------------------------------------

    /**
	 * Ajoute des données supplémentaires pour la vue sous la forme array(key => value)
     * 
	 * @access protected
     * @return array
	 */	
    protected function _add_more_data_for_add_record_view()
    {
        return array(
            'writer_rights' => $this -> _writer_rights, // Ne pas supprimer
            'doc_type' => $this -> _doc_infos, // Ne pas supprimer
            'private_groups' => $this -> _get_private_members_groups(),
            'qcm_id' => $this -> _qcm_id,
            'qcms' => $this -> _get_qcms_allowed(),
            'nb_answers' => $this -> _get_nb_answers(),
        );    
    }

	// --------------------------------------------------------------------

	/**
	 * retourne le nombre de réponses complétées
	 *
	 * @access	protected
	 * @return	integer Nombre de réponses complétées
	 */
    protected function _get_nb_answers()
    {        
        return count($this -> _form_params['answers']);           
    }

	// --------------------------------------------------------------------

	/**
	 * retourne la liste des qcm à gérer en tenant compte de la configuration du rédacteur
	 *
	 * @access	protected
	 * @return	array sous la forme array(id => name)
	 */
    protected function _get_qcms_allowed()
    {        
        $this -> CI -> load_addon_helper('qcm', 'qcm_questions_helper');
        
        return $this -> CI -> qcm_questions_helper -> get_qcm_allowed_for_manage_questions($this -> CI -> qcm_ids_model, $this -> _writer_rights);           
    }

	// --------------------------------------------------------------------

	/**
	 * retourne la liste des groupes membres privés
	 *
	 * @access	protected
	 * @return	array 
	 */
    protected function _get_private_members_groups()
    {
        return $this -> CI -> private_groups_helper -> get_all();
    }

	// --------------------------------------------------------------------

    /**
	 * Définit et retourne le layout de la vue
     * 
	 * @access protected
     * @return string nom du fichier qui définit le layout
	 */	
    protected function _get_layout_for_add_record_view()
    {
        return str_replace(ADMR_PUBLIC_ROOT_PATH, '..', QCM_VIEWS_PATH).'/add_qcm_questions_layout';
    }
    
	// --------------------------------------------------------------------

    /**
	 * Ajoute ou modifie un enregistrement
     * 
	 * @access protected
     * @return boolean
	 */	
    protected function _add_record()
    {
        return parent :: _add_record();
    }

	// --------------------------------------------------------------------

    /**
	 * Crée un nouvel enregistrement dans la base de donnée
     * 
	 * @access protected
     * @param array Tableau contenant les données de l'enregistrement
     * @return integer Identifiant de l'enregistrement créé
	 */
    protected function _insert_record($record)
    {
        $this -> CI -> load_addon_model('qcm', 'qcm_questions_ids_model', array('qcm_id' => $record['qcm_id']));

		$record['author_id'] = $this -> _get_writer_id();     
        $record['id'] = $this -> CI -> qcm_questions_ids_model -> insert($record);
        $this -> CI -> qcm_questions_model -> write($record);

        if (!ADMR_IS_ADMIN) {
 //           $this -> CI -> writer_qcm_model -> insert($record);    
        }
    
        return $record['id'];
    }
 
	// --------------------------------------------------------------------

    /**
	 * Met à jours les données de l'enregistrement dans la base de donnée
     * 
	 * @access protected
     * @param array Tableau contenant les données de l'enregistrement
     * @return void
	 */
    protected function _update_record($record)
    {
        if ($this -> CI -> qcm_questions_ids_model -> update($record)) {
            $this -> CI -> qcm_questions_model -> write($record);    
        };

        if (!ADMR_IS_ADMIN) {
 //           $this -> CI -> writer_qcm_model -> update($record);    
        }
    
        return TRUE;    
    }
    
	// --------------------------------------------------------------------

    /**
	 * Ajoute d'autres actions après l'ajout ou la mise à jour d'un document
     * 
	 * @access protected
     * @return void
	 */
    protected function _add_more_action_after_adding_record()
    {
        $params =& $this -> _form_params;

        // Nothing        
    }

	// --------------------------------------------------------------------
    
    /**
	 * Définit les valeurs des paramètres du formulaire
     * 
	 * @access protected
     * @param integer Id de l'enregistrement
     * @return array
	 */    
    protected function _set_form_params()
    {
        return array_merge(
            parent :: _set_form_params(),
            array(

            )
        );
    }

	// --------------------------------------------------------------------
    
    /**
	 * Récupère et renvoie les paramètres postés dans le formulaire
     * 
	 * @access protected
     * @return array Tableau contenant les paramètres postés dans le formulaire
	 */    
    protected function _get_form_params_posted()
    {
        $params = parent :: _get_form_params_posted();
        
        $params['answers'] = $this -> _get_answers_not_empty($params['answers']);

        if (!ADMR_IS_ADMIN) {
            $params = $this -> _apply_writer_rights_for_status_posted($params); // Prise en compte ou non du status pour le rédacteur uniquement       
        }
        
        return $params; // Paramètres postés dans le formulaire       
    }

	// --------------------------------------------------------------------

	/**
	 * retourne uniquement les réponses complétées
	 *
	 * @access	protected
     * @param array Tableau contenant toutes les réponses postées
	 * @return	array Tableau contenant les réponses complétées
	 */
    protected function _get_answers_not_empty($answers_posted)
    {        
        $answers = $answers_posted;
        
        foreach($answers as $key => $answer) {
            if (empty($answer['content1'])) {
                unset($answers[$key]);    
            }    
        }
        
        return $answers;           
    }

	// --------------------------------------------------------------------
    
    /**
	 * Vérifie et corrige si nécessaire le status du document postés en fonction des autorisations du rédacteur
     * 
	 * @access private
     * @param array Données postées
     * @return array Données vérifiées et postées
	 */    
    private function _apply_writer_rights_for_status_posted($params)
    {
        if (empty($this -> _writer_rights[$this -> _doc_infos['name'].'_act'])) {
            $params['status'] = $this -> _config['doc_status_by_default']; // Status du document par défaut          
        }
        
        return  $params;       
    }

	// --------------------------------------------------------------------
    
    /**
	 * Récupère et renvoie les données du document
     * 
	 * @access protected
     * @param integer Id de l'enregistrement
     * @return array Données enregistrées du document
	 */    
    protected function _get_record_data($record_id)
    {
        $this -> CI -> load_addon_model('qcm', 'qcm_questions_model');        

        $record_data = array_merge(
            $this -> CI -> qcm_questions_model -> read($record_id),
            array('status' => $this -> _config['doc_status_by_default'])
        );
        
        if (!empty($record_data['qcm_id'])) {
            $this -> CI -> load_addon_model('qcm', 'qcm_questions_ids_model', array('qcm_id' => $record_data['qcm_id']));
            
            $record_data = array_merge(
                $record_data,
                $this -> CI -> qcm_questions_ids_model -> get_by_id($record_id)
            );           
        }
        
        return $record_data;
    }

	// --------------------------------------------------------------------

    /**
	 * définit et renvoie les règles de validation des paramètres du formulaire
     * 
	 * @access protected
     * @param array Tableau contenant les paramètres à vérifer lors de la validation
     * @return array Tableau contenant les règles de validation des paramètres ainsi que les messages d'erreur associés 
     * sous la forme suivante : array(param1 => array(
     *                                               array('rules' => '', 'msg' => ''), // error 1
     *                                               array('rules' => '', 'msg' => ''), // error 2                                       
     *                                         ),
     *                                param2 => array(
     *                                               array('rules' => '', 'msg' => ''), // error 1
     *                                               array('rules' => '', 'msg' => ''), // error 2                                       
     *                                         ),
     *                               ...
     *                          )
	 */	
    protected function _get_form_params_rules($params)
    {
        return $this -> CI -> qcm_questions_model -> set_params_rules($params, $this -> _lang, $this -> _config);
    }

	// --------------------------------------------------------------------

	/**
	 * retourne l'url complète de la page
     * 
     * @access protected
	 * @return string Url complète de la page
	 */
    protected function _get_current_url()
    {
        return parent :: _get_current_url();
    }

	// --------------------------------------------------------------------

    /**
	 * Vérifie la validité d'accès au qcm par le rédacteur
     * 
	 * @access public
     * @param integer Identifiant du qcm
     * @return void
	 */
    protected function _is_qcm_allowed($qcm_id)
    {
        if (!empty($qcm_id) && ! array_key_exists($qcm_id, $this -> _get_qcms_allowed())) {
            return FALSE;
        }

        return TRUE;    
    }

	// --------------------------------------------------------------------

    /**
	 * Vérifie la validité d'accès au document par le rédacteur
     * 
	 * @access public
     * @param integer Identifiant de la question du qcm
     * @return void
	 */
    protected function _is_doc_allowed($doc_id)
    {   
        if (!empty($this -> _writer_rights[$this -> _doc_infos['name'].'_manage'])) // Autorisation d'accès à la section du document
        {
            if (empty($doc_id) && !empty($this -> _writer_rights[$this -> _doc_infos['name'].'_add'])) { // Ajout d'un nouveau document
                return TRUE;
            }
            else {  // Edition d'un document
                $doc = $this -> CI -> qcm_questions_ids_model -> get(array('id' => $doc_id, 'author_id' => $this -> _get_writer_id()));

                if (!empty($doc_id) && !empty($doc) && !empty($this -> _writer_rights[$this -> _doc_infos['name'].'_edit'])) {
                    return TRUE;
                }   
            }
        }
        
        return FALSE;    
    }

	// --------------------------------------------------------------------

 } // End of class