<?php
/**
 * Plugin Qcm (addon for plugin Adm_redacteurs)
 * Version  : 1.5 (2021/01/03)
 * Compatibility : Guppy v5.x et v6.00.00 et Adm_redacteurs v6.14+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

class Qcm_menuitems_model
{    
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    private $_DB_file_path = QCM_MENUITEMS_DB;  // Chemin du fichier contenant la base de données
    private $_data_DB = array();    // Tableau de données de la base de données    

    private $_params = array(
        'id_menuitem' => '', // Id de l'item de menu (clé primaire)
        'id_qcm' => '', // Id du document du plugin
    );

    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */
    public function __construct()
	{
        if (is_file($this -> _DB_file_path)) {
            $this -> _data_DB = $this -> read();    
        }       
	}

	// --------------------------------------------------------------------
 
	/**
	 * récupére les données des rédacteurs enregistrés (tous par défaut)
     * 
	 * @access public
     * @param  array Tableau de filtres (optionnel) sous la forme array($key1 => mixed $values1, $key2 => mixed $values2, ...)
     * @param string Option de tri (optionnel)
	 * @return array tableau contenant les données des rédacteurs enregistrés
	 */
	function get($filters = array(), $sort_by = '')
	{
        settype($filters, 'array');
        $data = array();
      
        if (!empty($filters) ) {
            $data = $this -> get_many_by_keys($filters);
        }
        else {
            $data = $this -> get_all();
        }
        
        if (!empty($sort_by)) {
            $data = $this -> sort_by_key($data, $sort_by);
        }
        
        return $data;
	}

	// --------------------------------------------------------------------

    /**
     * récupére tous les documents enregistrés
     * 
     * @access public
     * @return array
     */
    function get_all()
    {  
        return $this -> _data_DB;
    }

	// --------------------------------------------------------------------

    /**
     * récupérer le nombre de documents enregistrés
     * 
     * @access public
     * @param  string Type du document (optionnel)
     * @return integer
     */
	function count($filters = array())
	{
        settype($filters, 'array');
      
        if (!empty($filters) ) {
            return count($this -> get_many_by_keys($filters));
        }
        else {
            $data = $this -> count_all();
        }
 	}

	// --------------------------------------------------------------------

    /**
     * récupérer le nombre total de documents enregistrés
     * 
     * @access public
     * @return integer
     */
    function count_all()
    {
        return count($this -> _data_DB);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données identifié par la clé et sa valeur
     * 
     * @param array Données de la base
     * @param string Clé
     * @param string Valeur de la clé
     * @return array
     */
    function select_by_key($data_DB = array(), $key = '', $value = '')
    {
        settype($data_DB, 'array');
        settype($value, 'string');
        
        foreach($data_DB as $row) {
            if (isset($row[$key]) && $row[$key] == $value) {
                return $row;
            }
        }    
        
        return array();
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données identifié par la clé et sa ou ses valeurs
     * 
     * @param array Données de la base
     * @param string Clé
     * @param string|array Valeur(s) de la clé
     * @return array
     */
    function select_many_by_key($data_DB = array(), $key = '', $value = array())
    {
        settype($data_DB, 'array');
        settype($value, 'array');
        $data = array();
        
        foreach($data_DB as $row) {
            if (isset($row[$key]) && in_array($row[$key], $value)) {
                $data[$row['id']] = $row;
            }
        }    
        
        return $data;
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données des documents identifiés par les critères de sélection
     * 
     * @param array Tableau contenant les critères de sélection sous la forme array( key => value)
     * @return array
     */
    function get_many_by_keys($keys)
    {
        settype($keys, 'array');

        return $this -> select_many_by_many_keys($this -> _data_DB, $keys);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données identifié par la clé et sa ou ses valeurs
     * 
     * @param array Données de la base
     * @param array Tableau contenant les critères de sélection sous la forme array($key1 => mixed $values1, $key2 => mixed $values2,...)
     * @return array
     */
    function select_many_by_many_keys($data_DB = array(), $keys = array())
    {
        settype($data_DB, 'array');
        settype($keys, 'array');

        
        $data = $data_DB;

        foreach($keys as $key => $values) {
            
            if (empty($values)) {
                continue;    
            }
            
            settype($values, 'array');
            
            foreach($data as $id => $row) {
                if (!isset($row[$key]) || !in_array($row[$key], $values)) {
                    unset($data[$id]);
                }
            }           
        }        
        
        return $data;
    }

	// --------------------------------------------------------------------

    /**
     * récupére les données brutes de la table indexée des documents (id||status||type||title||comment||author)
     *
     * @return array
     */
    private function read()
    {
        $data_DB = array();
        $records = array();
        
        if (is_file($this -> _DB_file_path)) {
            $data_DB = file($this -> _DB_file_path);
    		
            foreach($data_DB as $data) {
                list($id_menuitem, $id_qcm) = explode(CONNECTOR, trim($data));
    			
                $records[$id_menuitem] = array(
                    'id_menuitem' => $id_menuitem, // Id de l'item de menu
                    'id_qcm' => $id_qcm, // Id du document
                );
            }
        }
        
        return $records;
    }

	// --------------------------------------------------------------------

    /**
     * Efface le contenu de la table et le remplace avec les données passées en paramètre
     * 
     * @param array Données à écrire
     * @return array
     */
    private function write()
    {   
        $data = $this -> _data_DB;
        $file_path = $this -> _DB_file_path;
        $data_out = '';
        
        foreach($data as $row) {     
            $data_out .= implode(CONNECTOR, array_values($row))."\n";
        }

        $fhandle = fopen($file_path, 'w');
        fputs($fhandle, $data_out);
        fclose($fhandle);
        
        return TRUE;
    }

	// --------------------------------------------------------------------

    /**
     * Ajoute un enregistrement dans la table indexée des documents (id||status||type||title||comment||author)
     * 
     * @param array Données de l'enregistrement du rédacteur dans la table
     * @return integer Id du compteur
     */
    function insert($data)
    {
        if (empty($data) || empty($data['id_menuitem']) || isset($this -> _data_DB[$data['id_menuitem']])) {
            return FALSE;
        }
        
        $this -> _data_DB[$data['id_menuitem']] = array_merge($this -> _params, $this -> _clean($data), array('id_menuitem' => $data['id_menuitem']));
        
        $this -> write();

        return true;
    }
    
	// --------------------------------------------------------------------

    /**
     * Met à jour l'enregistrement identifié par le paramètre id 
     * dans la table indexée des documents (id||status||type||title||comment||author)
     * 
     * @param array Données de l'enregistrement du document à mettre à jours dans la table
     * @return array
     */
    function update($data)
    {
        if (empty($data) || empty($data['id_menuitem']) || !isset($this -> _data_DB[$data['id_menuitem']])) {
            return FALSE;
        }
        
        $this -> _data_DB[$data['id_menuitem']] = array_merge($this -> _params, $this -> _clean($data));
        
        $this -> write();

        return TRUE;
    }

	// --------------------------------------------------------------------

    /**
     * supprime les documents identifiés par les identifiants des items de menu dans la table
     * 
     * @param integer|array Identifiants des items de menu dans la table
     * @return array
     */
    function delete_by_menuitem_id($ids)
    {
        settype($ids, 'array');
        $db =& $this -> _data_DB;
        
        foreach($db as $key => $row){
            if (in_array($row['id_menuitem'], $ids)) {
                unset($db[$key]);
            }            
        }
            
        $this -> write();
        
        return TRUE;
    }

	// --------------------------------------------------------------------

    /**
     * supprime les documents identifiés par les identifiants des documents du plugin dans la table
     * 
     * @param integer|array Identifiants des documents du plugin dans la table
     * @return array
     */
    function delete_by_qcm_id($ids)
    {
        settype($ids, 'array');
        $db =& $this -> _data_DB;
        
        foreach($db as $key => $row){
            if (in_array($row['id_qcm'], $ids)) {
                unset($db[$key]);
            }            
        }
            
        $this -> write();
        
        return TRUE;
    }

	// --------------------------------------------------------------------
 
	/**
	 * Nettoie les données du rédacteur
     * 
     * @access private 
	 * @param array Données du rédacteur à nettoyer
	 * @return array
	 */        
    private function _clean($data)
    {
        settype($data, 'array');
        
        return array_map('RemoveConnector', array_intersect_key($data, $this -> _params));
    }

	// --------------------------------------------------------------------

    /**
     * Tri les enregistrements
     * 
     * @param string Option de tri
     * @return array Données triées
     */
    function sort_by_key($data, $sort_option)
    {       
        if(empty($data)) {
            return $data;
        }

        // Tableau en colonne
    /*    foreach ($data as $key => $row) {
            $id_qcm[$key] = $row['id_qcm'];
            $id_menuitem[$key] = $row['id_menuitem'];
        }
        //
        
        switch ($sort_option) {
			case 'id_menuitem_desc' :
				array_multisort($id_menuitem, SORT_NUMERIC, SORT_DESC, $data);
				break;
			case 'id_menuitem_asc' :
				array_multisort($id_menuitem, SORT_NUMERIC, SORT_ASC, $data);
				break;
			case 'id_qcm_desc' :
				array_multisort($id_qcm, SORT_NUMERIC, SORT_DESC, $data);
				break;
			case 'id_qcm_asc' :
				array_multisort($id_qcm, SORT_NUMERIC, SORT_ASC, $data);
				break;
			default :
				array_multisort($id_menuitem, SORT_NUMERIC, SORT_DESC, $data);
		}*/	
	
		return $data;
    }

	// --------------------------------------------------------------------

} // End of Class